'use strict';

const Promise   = require('bluebird');
const eventBus  = require('@codefresh-io/eventbus');
const monitor   = require('@codefresh-io/cf-monitor');
const logger    = require('cf-logs').Logger("codefresh:infra:eventbus");
const config    = require('./../config');


class Eventbus {

    constructor() {
        this.eventbusInitialized = false;
    }

    /**
     * starts the connection to eventbus
     * @returns {*}
     */
    init(config) {
        return Promise.resolve()
            .then(() => {
                this.config = config;

                var deferred = Promise.defer();

                //TODO a fallback for case where rabbitmq is not up. should be removed once rabbitmq is fully used
                setTimeout(() => {
                    deferred.resolve();
                }, 30000);

                eventBus.init({
                    bus: {
                        url: this.config.eventbus.uri,
                        reconnectInterval: 5
                    },
                    store: {
                        host: this.config.postgres.host,
                        database: this.config.postgres.database,
                        user: this.config.postgres.user,
                        password: this.config.postgres.password,
                        port: this.config.postgres.port,
                    },
                    microServiceName: this.config.eventbus.serviceName
                });

                eventBus.on('ready', () => {
                    console.log('Eventbus ready');
                    this.eventbusInitialized = true;
                    deferred.resolve();
                });

                eventBus.on('error', (err) => {
                    const error = new Error(`Eventbus error: ${err.stack}`);
                    logger.error(error.stack);
                    monitor.noticeError(error);
                });

                return deferred.promise;
            })
            .then(() => {
                return this._start();
            });
    }


    /**
     * stops the connection to eventbus
     * @returns {*}
     */
    stop() {
        if (!this.eventbusInitialized) {
            return Promise.resolve();
        }

        var deferred = Promise.defer();

        eventBus.on('close', () => {
            logger.info('Eventbus client closed');
            deferred.resolve();
        });

        eventBus.quit();

        return deferred.promise;
    }

    _start() {}

    publishEvent(name, data) {
        data.originator = data.originator || config.name;
        logger.debug(`Publishing event: ${name}`);
        eventBus.publish(name, data)
            .catch((err) => {
                logger.error(err.stack);
                monitor.noticeError(err);
            })
            .done();
    }

}


module.exports = new Eventbus();
