'use strict';

const _ = require('lodash');
const Promise = require('bluebird');
const Base = require('../../../kube-native/resource.base.controller');
const ReleasePool = require('./pool');
const CFError = require('cf-errors');

class ReleaseController extends Base {

    _getRelevantSecrets(name, revisions = []) {
        const cmCtl = this._kubeManager.getSecretController(this._ns);

        if (name && revisions.length) {
            return Promise.map(revisions, (revision) => {
                return cmCtl.get({ labelSelector: `version=${revision},name=${name}` });
            })
                .then(_.flattenDeep)
                .then(_.compact);
        }

        if (name) {
            return cmCtl.get({ labelSelector: `name=${name}` });
        }

        return cmCtl.get({ labelSelector: 'owner=helm' });
    }

    /**
     *
     * @param actualVersions - it is revision that we need start from for retrieval,
     * it is part of optimization, we not need retrieve revisions if we already have it,
     * { datadog: 1, demochat: 2}
     * @returns {*}
     */
    getWithOptimization(actualVersions = {}) {
        return this._getRelevantSecrets()
        .then(async (secrets) => {
            const pool = new ReleasePool({ secrets });
            await pool.prepareReleasesWithOptimization(actualVersions);
            const releases = await pool.getAllReleases();
            return { releases, ignoredReleases: pool.getIgnoredRevisions(), deletedReleases: pool.getDeletedReleases(_.keys(actualVersions)) };
        });
    }

    get() {
        return this._getRelevantSecrets()
            .then((secrets) => {
                const pool = new ReleasePool({ secrets });
                return pool.prepareReleases()
                    .then(() => pool.getAllReleases());
            });
    }

    describe(name, revisions = []) {
        return this._getRelevantSecrets(name, revisions)
            .then((secrets) => {
            if(!secrets || _.isEmpty(secrets)){
                throw new CFError({
                    message: `Release ${name} not found`,
                    recognized: true
                });
            }
            const pool = new ReleasePool({ secrets });
            return pool.prepareReleases()
            .then(() => pool.getReleaseFromPool(name))
        });
    }

    create(resourceDescriptor) {
        throw new CFError('Not supported yet');
    }

    delete(name) {
        throw new CFError('Not supported yet');
    }

    update(name, resourceDescriptor) {
        throw new CFError('Not supported yet');
    }

    publishEvent(name, data) {
        this.emit(name, data);
    }

    compareRevisionsInRelease(releaseName, base, head){
        return this.describe(releaseName, [base, head])
        .then(release => release.compareRevisions(base, head));
    }

    getChartManifestForRevision(name, revision){
        return this.describe(name, revision && [revision])
        .then(release => release.getRevisionByVersion(revision))
        .then(revision => revision.getManifest())
    }

    getChartValuesForRevision(name, revision){
        return this.describe(name, revision && [revision])
        .then(release => release.getRevisionByVersion(revision))
        .then(revision => revision.getValues())
    }
    getChartHooksForRevision(name, revision){
        return this.describe(name, revision && [revision])
        .then(release => release.getRevisionByVersion(revision))
        .then(revision => revision.getHooks())
    }

    getChartDescriptorForRevision(name, revision){
        return this.describe(name, revision && [revision])
        .then(release => release.getRevisionByVersion(revision))
        .then(revision => revision.getChart())
    }

}

module.exports = ReleaseController;
