'use strict';

const chai = require('chai');
const sinonChai = require('sinon-chai');
const _ = require('lodash');
const proxyquire = require('proxyquire');
const Revision = require('./../revision');
const Release = require('../release')
const config = require('../../../../../config');

const expect = chai.expect;
chai.use(sinonChai);

describe('revision', () => {
    function getFakeDataToInitiateRevision(){
        return {
            NAME: 'release-name',
            STATUS: 'DEPLOYED',
            VERSION: '1',
            MODIFIED_AT: 'some-time',
            CREATED_AT: 'some-time',
            dataAsBase64: ''
        };
    }

    function getProxyRevision() {
        return proxyquire('./../revision.js', {
            'zlib': {
                gunzip(data, cb) {
                    cb(null, new Buffer('fake'));
                }
            },
        });
    }

    it('should create new release', () => {
        const r = new Revision(getFakeDataToInitiateRevision());
        expect(r).to.have.all.keys(['_name', '_createdAt', '_modifiedAt', '_rawData', '_status', '_version']);
    });

    it('should init revision', () => {
        const RevisionProxy = getProxyRevision();
        const r = new RevisionProxy(getFakeDataToInitiateRevision());
        r.init()
        .then(() => {
            expect(r.decodedObject).to.exist;
        });
    });

    it('should correctly update keys, with dot support in key', (done) => {
        const RevisionProxy = proxyquire('./../revision.js', {
            'zlib': {
                gunzip(data, cb) {
                    cb(null, new Buffer(JSON.stringify({
                        info: {
                            first_deployed: 0,
                            last_deployed: 0,
                            Description: 'hello',
                        },
                        config: {
                          'test.test': {
                              test: true
                          }
                        }
                    })));
                }
            },
        });
        const r = new RevisionProxy(getFakeDataToInitiateRevision());
        r.init()
            .then(r.getValues.bind(r))
            .then((values) => {
                expect(values).to.deep.eq({
                    "data": "test.test:\n  test: true\n"
                });
                done()
            });
    });
});

describe('Release', () => {
    it('Should build limited history', () => {
        const firstRelease = {
            name: 'rel',
            version: 5,
            _decodedObject: { chart: { metadata: { name: 'chartName' } }, namespace: 'ns' }
        };
        const revisions = [8, 4, 10, 30, 45, 46, 42, 50, 49, 41, 20, 43, 44, 47, 48, 11];
        const release = new Release(firstRelease, 'tillerNs');
        config.helm.maxHelmHistorySize = 10;
        _.forEach(revisions, r => {
            release.addRevision({ version: r });
        });
        expect(_.orderBy(_.map(release._history, r => r.version))).to.eql([41,42,43,44,45,46,47,48,49,50]);
    });
    it('Should build full history', () => {
        const firstRelease = {
            name: 'rel',
            version: 5,
            _decodedObject: { chart: { metadata: { name: 'chartName' } }, namespace: 'ns' }
        };
        const revisions = [8, 4, 10, 30, 45, 46, 42, 50, 49, 41, 20, 43, 44, 47, 48, 11];
        const release = new Release(firstRelease, 'tillerNs');
        config.helm.maxHelmHistorySize = undefined;
        _.forEach(revisions, r => {
            release.addRevision({ version: r });
        });
        expect(_.orderBy(_.map(release._history, r => r.version))).to.eql([4,5,8,10,11,20,30,41,42,43,44,45,46,47,48,49,50]);
    });
})