'use strict';

const _                      = require('lodash');
const Promise                = require('bluebird');
const YAML                   = require('js-yaml');
const crypto                 = require('crypto');
const logger                 = require('cf-logs').Logger('codefresh:lib:kube-native:secret:secret-controller');
const ResourceController     = require('./../resource.base.controller');
const Secret                 = require('./secret');
const ImagePullSecretBuilder = require('./image.pull.secret.builder');
const SecretEvents           = require('./secret.events');
const secretLabels           = require('./secret.labels');



class KubeSecretController extends ResourceController {

    constructor(data) {
        super(data);
    }

    createImagePullSecret({ credentials, domain, ns, name, labels }) {

        labels                 = _.merge(labels, {
            [secretLabels.IMAGE_PULL_SECRET_HASH.key]: this.buildImagePullSecretHash({
                username: credentials.username,
                password: credentials.password
            }),

        });
        const secretDescriptor = {
            name,
            credentials,
            domain,
            labels
        };

        //TODO use the resouceController create function
        const deferred    = Promise.defer();
        const isYamlBased = !!_.get(secretDescriptor, 'yaml', false);
        let secretTemplate;
        if (isYamlBased) {
            secretTemplate = YAML.load(secretDescriptor.yaml);
        } else {
            secretTemplate = new ImagePullSecretBuilder(secretDescriptor).build();
        }
        this.getClient()
            .post({ body: secretTemplate })
            .then((secret) => {
                const sec = new Secret(secret, { selector: this.getSelector() });

                this.publishEvent(SecretEvents.types.ADDED, {
                    name: sec.getName()
                });
                return deferred.resolve(sec);
            })
            .catch(deferred.reject.bind(deferred));

        return deferred.promise;
    }

    buildImagePullSecretName({ domain, name, ns }) {
        return `codefresh-generated-${domain}-${name}-${ns}`;
    }

    buildImagePullSecretHash({ username, password }) {
        return crypto.createHash(
            'sha256')
            .update(`${username}:${password}`)
            .digest('hex').substring(0, 62)
    }

    save({ domain, ns, name, credentials, labels }) {
        const candidate = this.buildImagePullSecretName({ domain, ns, name });
        const opt = {
            domain,
            credentials,
            name: candidate,
            ns,
            labels,
        };
        const ctrl = this._kubeManager
            .getSecretController
            .bind(this._kubeManager);
        return ctrl(ns)
            .describe(candidate)
            .catch((err) => {
                //check error for not found
                if (err.code === 404) {
                    return;
                }
                throw err;
            })
            .then((secret) => {
                if (!secret) {
                    return ctrl(ns)
                        .createImagePullSecret(opt);
                } else {
                    const expectedHash = ctrl(ns)
                        .buildImagePullSecretHash({
                            username: credentials.username,
                            password: credentials.password,
                        });
                    const currentHash = secret.getLabelByKey(secretLabels.IMAGE_PULL_SECRET_HASH.key);
                    if (currentHash === expectedHash) {
                        return Promise.resolve(secret);
                    } else {
                        return ctrl(ns)
                            .delete(candidate)
                            .catch((err) => {
                                // some error occured, continure anyway on creation process
                                // TODO: send err to monitor
                                logger.error(err.stack);
                                return;
                            })
                            .then(() => {
                                return ctrl(ns)
                                    .createImagePullSecret(opt);
                            });
                    }
                }
            });

    }


}

module.exports = KubeSecretController;
