'use strict';

const _               = require('lodash');
const Promise         = require('bluebird');
const ResponseHandler = require('./../helpers/ResponseHandler');

class CrudController {

    constructor(data) {
        this._shouldLookForNs = _.get(data, 'lookForNamespaces', false);
    }

    getAll(request, response, next) {
        const filters                = request.filters;
        const kubeResourceController = request.kubeResourceController;
        const labelSelector = request.query.labelSelector;
        const searchFilter = labelSelector ? { labelSelector } : {};
        if (this._shouldLookForNs) {
            const namespaces = request.namespaces;
            Promise.map((namespaces), (ns) => {
                return kubeResourceController(ns)
                    .get(searchFilter)
                    .map(resource => resource.getBasicData());
            })
                .then(_.flattenDeep)
                .then((data) => this._filter(data, filters))
                .then(ResponseHandler.respondWithResult(response))
                .done(next, next);
        } else {
            kubeResourceController()
                .get()
                .map(resource => resource.getBasicData())
                .then((data) => this._filter(data, filters))
                .then(ResponseHandler.respondWithResult(response))
                .done(next, next);
        }

    }

    describe(request, response, next) {
        const kubeResourceController = request.kubeResourceController;
        let omitFields               = _.get(request, 'query.omit', ['']);
        if (_.isString(omitFields) && _.includes(omitFields, ',')) {
            omitFields = omitFields.split(',');
        }
        if (this._shouldLookForNs) {
            const namespaces = request.namespaces;
            Promise.map((namespaces), (ns) => {
                return kubeResourceController(ns)
                    .describe(request.params.name)
                    .then(resource => resource.getFullData());
            })
                .then((data) => {
                    if (_.size(namespaces) === 1) {
                        return _.omit(_.head(data), omitFields);
                    }
                    return _.omit(data, omitFields);
                })
                .then(ResponseHandler.respondWithResult(response))
                .done(next, next);
        } else {
            kubeResourceController()
                .describe(request.params.name)
                .then(resource => resource.getFullData())
                .then((data) => {
                    return _.omit(data, omitFields);
                })
                .then(ResponseHandler.respondWithResult(response))
                .done(next, next);
        }
    }

    create(request, response, next) {
        const kubeResourceController = request.kubeResourceController;
        const data                   = _.get(request, 'body', {});
        if (this._shouldLookForNs) {
            const namespaces = request.namespaces;
            Promise.map((namespaces), (ns) => {
                return kubeResourceController(ns)
                    .create(data)
                    .then(resource => resource.getBasicData());
            })
                .then((data) => {
                    if (_.size(namespaces) === 1) {
                        return _.head(data);
                    }
                    return data;
                })
                .then(ResponseHandler.respondWithResult(response, 201))
                .done(next, next);
        } else {
            kubeResourceController()
                .create(data)
                .then(resource => resource.getBasicData())
                .then(ResponseHandler.respondWithResult(response, 201))
                .done(next, next);
        }
    }

    delete(request, response, next) {
        const kubeResourceController = request.kubeResourceController;
        if (this._shouldLookForNs) {
            const namespaces = request.namespaces;
            return Promise.map((namespaces), (ns) => {
                return kubeResourceController(ns)
                    .delete(request.params.name);
            })
                .then((data) => {
                    if (_.size(namespaces) === 1) {
                        return _.head(data);
                    }
                    return data;
                })
                .then(ResponseHandler.respondWithResult(response, 204))
                .done(next, next);
        } else {
            kubeResourceController()
                .delete(request.params.name)
                .then(ResponseHandler.respondWithResult(response, 204))
                .done(next, next);
        }
    }

    update(request, response, next) {
        const kubeResourceController = request.kubeResourceController;
        const data                   = _.get(request, 'body', {});
        if (this._shouldLookForNs) {
            const namespaces = request.namespaces;
            Promise.map((namespaces), (ns) => {
                return kubeResourceController(ns)
                    .update(request.params.name, data)
                    .then(resource => resource.getBasicData());
            })
                .then((data) => {
                    if (_.size(namespaces) === 1) {
                        return _.head(data);
                    }
                    return data;
                })
                .then(ResponseHandler.respondWithResult(response, 200))
                .done(next, next);
        } else {
            kubeResourceController()
                .update(request.params.name, data)
                .then(resource => resource.getBasicData())
                .then(ResponseHandler.respondWithResult(response, 200))
                .done(next, next);
        }
    }

    _filter(data, filters) {
        if (!_.isArray(data)) {
            data = [data];
        }

        return Promise.map(data, (obj) => {
            let filtredObj = _.cloneDeep(obj);
            let stop       = false;
            _.forEach(filters, (filterInstance) => {
                const res = filterInstance.filter(obj);
                if (res && !stop) {
                    filtredObj = _.merge(filtredObj, res);
                } else {
                    stop       = true;
                    filtredObj = undefined;
                }
            });
            return filtredObj;

        })
            .then(_.flattenDeep)
            .then(_.compact);
    }
}

module.exports = CrudController;
