'use strict';

const _                      = require('lodash');
const Promise                = require('bluebird');
const ResponseHandler        = require('./../../../helpers/ResponseHandler');
const BaseCrudController     = require('./../../crud.base.controller');
const supportedKubeResources = require('./../../../../lib/kube-native/supported.recourses');
const controller             = require('./../../../../lib/internal-components/project/controller');
const CFError = require('cf-errors');
class Controller extends BaseCrudController {

    constructor() {
        super({
            lookForNamespaces: true
        });
    }

    getAll(request, response, next) {
        const kubeManager = request.kubeManager;
        const namespaces  = request.namespaces;
        Promise.map(namespaces, (ns) => {
            return controller.getAll({ kubeManager, ns })
        })
            .then(_.flattenDeep)
            .map((project) => project.toMinifiedJson())
            .then(_.flattenDeep)
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);

    }

    descibe(request, response, next) {
        const kubeManager = request.kubeManager;
        const namespaces  = request.namespaces;
        Promise.map((namespaces), (ns) => {
            return controller.describe({ kubeManager, ns, name: request.params.name });
        })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data).toJson();
                }
                return data.map((obj) => obj.toJson());
            })
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);
    }

    create(request, response, next) {
        const kubeManager = request.kubeManager;
        const namespaces  = request.namespaces;
        const body        = _.get(request, 'body', {});
        this._validateBodyStructure(body)
            .then(() => {
                return Promise.map((namespaces), (ns) => {
                    return controller.create({ kubeManager, ns, data: body });
                });
            })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data).toJson();
                }
                return data.map((obj) => obj.toJson());
            })
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);
    }

    delete(request, response, next) {
        const kubeManager = request.kubeManager;
        const namespaces  = request.namespaces;
        Promise.map((namespaces), (ns) => {
            return controller.describe({ kubeManager, ns, name: request.params.name });
        })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.compact(data);
                }
                return data;
            })
            .map(project => controller.delete({ kubeManager, project }))
            .then(ResponseHandler.respondWithResult(response, 204))
            .done(next, next);
    }

    /**
     * Expecting body to have to following structure
     * { any-supported-resource-name: Object | String | Object[] | String[]}
     * For example:
     * {deployment: {kind: Deployment}, {service: ['yaml for service', {kind: Service}]}
     */
    update(request, response, next) {
        const namespaces  = request.namespaces;
        const kubeManager = request.kubeManager;
        const body        = _.get(request, 'body', {});

        this._validateBodyStructure(body)
            .then(() => {
                return Promise.map((namespaces), (ns) => {
                    return controller.update({
                        kubeManager,
                        ns,
                        name: request.params.name,
                        data: body
                    });
                });
            })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data).toJson();
                }
                return data.map((obj) => obj.toJson());
            })
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);
    }

    /**
     * Look at the supported resources (supportedKubeResources) alias validate types
     * @param data
     * @private
     */
    _validateBodyStructure(data) {
        return Promise
            .map(_.keys(data), (candidate) => {
                let supported = false;
                return Promise.map(_.keys(supportedKubeResources), (resource) => {
                    const resourceDescriptor = supportedKubeResources[resource];
                    if (!supported) {
                        supported        = _.includes(resourceDescriptor.alias, candidate);
                        let shouldRename = _.includes(resourceDescriptor.alias, candidate) &&
                                           !(resourceDescriptor.name === candidate);
                        if (shouldRename) {
                            data[resourceDescriptor.name] = data[candidate];
                            _.unset(data, candidate);
                        }
                    }
                })
                    .then(() => {
                        if (!supported) {
                            throw new CFError({
                                message: `Resource ${candidate} not supported`,
                                recognized: true,
                            });
                        }
                    });
            })
            .return(data);
    }

}

module.exports = new Controller();
