'use strict';

const _                  = require('lodash');
const Promise            = require('bluebird');
const ResponseHandler    = require('./../../../helpers/ResponseHandler');
const BaseCrudController = require('./../../crud.base.controller');
const StatusAnalyzer     = require('./../../../../lib/status.analyzer.base');

function formatQueryAsString(queryAsStringOrArryay) {
    if (_.isArray(queryAsStringOrArryay)) {
        return queryAsStringOrArryay.join(',');
    }
    return queryAsStringOrArryay;
}

class Controller extends BaseCrudController {

    constructor() {
        super({
            lookForNamespaces: true
        });
    }

    getAll(request, response, next) {
        const kubeManager          = request.kubeManager;
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
        const podController        = kubeManager.getPodController.bind(kubeManager);
        const labelSelector = formatQueryAsString(request.query.labels);
        const namespaces = request.namespaces;
        Promise.map((namespaces), (ns) => {

            return serviceController(ns)
                .get({
                    labelSelector
                })
                .map((svc) => {
                    const serviceSelector = svc.getServiceSelector();
                    if (!_.isEmpty(serviceSelector)) {
                        return Promise.map(_.keys(serviceSelector), (key) => {
                            return `${key}=${serviceSelector[key]}`;
                        })
                            .then((selector) => selector.join(','))
                            .then((selector) => {
                                return Promise.all([
                                    deploymentController(ns)
                                        .get({
                                            labelSelector: selector
                                        })
                                        .then(_.head),
                                    replicasetController(ns)
                                        .get({
                                            labelSelector: selector
                                        })
                                        .map(rs => rs.getReplicas() !== 0 ? rs : undefined)
                                        .then(_.compact),
                                    podController(ns)
                                        .get({
                                            labelSelector: selector
                                        }),
                                ]);
                            })
                            .then(([deployment, rsArray, podArray]) => {
                                const analyzedStatus = this._analyzeStatus({
                                    svc,
                                    deployment,
                                    rsArray,
                                    podArray
                                });
                                return _.merge(svc.getBasicData(),
                                    {
                                        globalStatus: analyzedStatus,
                                        deployment: deployment && deployment.getBasicData()
                                    })
                            })
                            .catch(() => {
                                // todo: report to new rellic
                                const analyzedStatus = this._analyzeStatus({
                                    svc
                                });
                                return _.merge(svc.getBasicData(),
                                    {
                                        globalStatus: analyzedStatus,
                                    });
                            });
                    } else {
                        const analyzedStatus = this._analyzeStatus({
                            svc
                        });
                        return _.merge(svc.getBasicData(),
                            {
                                globalStatus: analyzedStatus,
                            });
                    }
                });
        })
            .then(_.flattenDeep)
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);
    }

    describe(request, response, next) {
        const kubeManager          = request.kubeManager;
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
        const podController        = kubeManager.getPodController.bind(kubeManager);

        const namespaces = request.namespaces;
        Promise.map((namespaces), (ns) => {
            return serviceController(ns)
                .describe(request.params.name)
                .then((svc) => {
                    const serviceSelector = svc.getServiceSelector();
                    if (!_.isEmpty(serviceSelector)) {
                        return Promise.map(_.keys(serviceSelector), (key) => {
                            return `${key}=${serviceSelector[key]}`;
                        })
                            .then((selector) => selector.join(','))
                            .then((selector) => {
                                return Promise.all([
                                    deploymentController(ns)
                                        .get({
                                            labelSelector: selector
                                        })
                                        .then(_.head)
                                        .then((deploy) => {
                                            if (deploy) {
                                                svc.setDeployment(deploy.getName());
                                            }
                                            return deploy;
                                        }),
                                    replicasetController(ns)
                                        .get({
                                            labelSelector: selector
                                        })
                                        .map(rs => rs.getReplicas() !== 0 ? rs : undefined)
                                        .then(_.compact),
                                    podController(ns)
                                        .get({
                                            labelSelector: selector
                                        }),
                                ]);
                            })
                            .then(([deployment, rsArray, podArray]) => {
                                const analyzedStatus = this._analyzeStatus({
                                    svc,
                                    deployment,
                                    rsArray,
                                    podArray
                                });
                                return _.merge(svc.getFullData(),
                                    {
                                        globalStatus: analyzedStatus
                                    })
                            });
                    } else {
                        const analyzedStatus = this._analyzeStatus({
                            svc
                        });
                        return _.merge(svc.getFullData(),
                            {
                                globalStatus: analyzedStatus
                            });
                    }
                })
        })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data);
                }
                return data;
            })
            .then(ResponseHandler.respondWithResult(response))
            .done(next, next);

    }

    delete(request, response, next) {
        const kubeManager          = request.kubeManager;
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
        const podController        = kubeManager.getPodController.bind(kubeManager);
        const namespaces           = request.namespaces;
        const cascade              = _.get(request, 'query.cascade', true);
        Promise.map((namespaces), (ns) => {
            return serviceController(ns)
                .describe(request.params.name)
                .then((svc) => {
                    if (cascade) {
                        const serviceSelector = svc.getServiceSelector();
                        return Promise.map(_.keys(serviceSelector), (key) => {
                            return `${key}=${serviceSelector[key]}`
                        })
                            .then(_.head)
                            .then((selector) => {
                                return Promise.all([
                                    deploymentController(ns).get({ labelSelector: selector }),
                                    replicasetController(ns).get({ labelSelector: selector }),
                                    podController(ns).get({ labelSelector: selector }),
                                ]);
                            })
                            .then(([deployments, replicasets, pods]) => {
                                return Promise.resolve()
                                    .then(() => this._cascadeDeletationTo(deploymentController,
                                        ns,
                                        deployments))
                                    .then(() => this._cascadeDeletationTo(replicasetController,
                                        ns,
                                        replicasets))
                                    .then(() => this._cascadeDeletationTo(podController, ns, pods));
                            })
                            .catch((err) => {
                                throw err;
                            })
                    } else {
                        return svc;
                    }
                })
                .then(() => {
                    return serviceController(ns).delete(request.params.name);
                });

        })
            .then((data) => {
                if (_.size(namespaces) === 1) {
                    return _.head(data);
                }
                return data;
            })
            .then(ResponseHandler.respondWithResult(response, 204))
            .done(next, next);
    }

    _cascadeDeletationTo(controller, ns, resources = []) {
        return Promise.map(resources, (resource) => {
            return controller(ns).delete(resource.getName())
                .catch((err) => {
                    // do not stop the execution if one resource delete request throws an error
                    // todo: think of how to return this information back to the client
                    console.log(err.stack);
                    return;
                });
        });
    }

    /**
     * Analyze the total status of the svc,deploy,rs,pod set
     * @param svc
     * @param deployment
     * @param rsArray
     * @param podArray
     * @return {{type: string, resources: {svc: *, deployment: *, replicaSets, pods}}
     * @private
     */
    _analyzeStatus({ svc, deployment, rsArray, podArray }) {
        const status = {
            type: StatusAnalyzer.types.unknown.status,
            resources: {
                svc: svc.getEntityStatus(),
                deployment: deployment && deployment.getEntityStatus(),
                replicaSets: rsArray && rsArray.map(rs => rs.getEntityStatus()),
                pods: podArray && podArray.map(pod => pod.getEntityStatus())
            }
        };

        if (svc.getEntityStatus().status !== StatusAnalyzer.types.suceess.status) {
            status.type    = StatusAnalyzer.types.failed;
            status.message = 'Kubernetes service not available';
            return status;
        }

        if (_.size(rsArray) > 1) {
            let rs = _.find(rsArray, (obj) => {
                return obj.getEntityStatus().status !== StatusAnalyzer.types.suceess.status;
            });
            if (!rs) {
                rs = _.head(rsArray);
            }
            const key          = rs.getLabelByKey('pod-template-hash');
            const pods         = _.filter(podArray, (obj) => {
                return obj.getLabelByKey('pod-template-hash') === key;
            });
            const pod          = _.head(pods);
            const podStatusObj = pod.getEntityStatus();
            status.type        = podStatusObj.status;
            status.message     = podStatusObj.message;

        } else {
            const pod = _.find(podArray,
                (po) => po.getEntityStatus().status !== StatusAnalyzer.types.suceess.status);
            if (pod) {
                const podStatusObj = pod.getEntityStatus();
                status.type        = podStatusObj.status;
                status.message     = podStatusObj.message;
            } else {
                status.type = StatusAnalyzer.types.suceess.status;
            }
        }
        return status;
    }

}

module.exports = new Controller();
