'use strict';

const _              = require('lodash');
const middlewares    = require('./../../infra/middleware');
const BaseController = require('./crud.base.controller');

const supportedRoutes = {
    'namespaces': {
        kubeController: 'namespace',
        resolveNamespacesFromQuery: false
    },
    'services': {
        expressController: {},
        kubeController: 'service'
    },
    'pod': {
        baseController: {
            lookForNamespaces: true
        },
        kubeController: 'pod'
    },
    'deployments': {
        kubeController: 'deployment',
        baseController: {
            lookForNamespaces: true
        },
    },
    'configmaps': {
        baseController: {
            lookForNamespaces: true
        },
        kubeController: 'configmap',
    },
    'secrets': {
        expressController: {
            extraRoutes: true
        },
        kubeController: 'secret'
    },
    'replicasets': {
        kubeController: 'replicaset'
    },
    'nodes': {
        kubeController: 'node'
    },
    'releases': {
        kubeController: 'release',
        expressController: {
            extraRoutes: true
        },
    },
    'releases-helm3': {
        kubeController: 'releaseHelm3',
        expressController: {
            extraRoutes: true,
            path: 'releases'
        },
    }
};

function initExpressController(route, options) {
    const router = require('express').Router();
    let controller;

    // non standard controller
    if (options.expressController) {
        controller = require(`./kube-native/${options.expressController.path || route}/controller`);
    } else {
        controller = new BaseController({
            lookForNamespaces: _.get(options, 'baseController.lookForNamespaces', false)
        });
    }
    if (!options.resolveNamespacesFromQuery) {
        router.use(middlewares.resolveNamespacesFromQuery);
    }

    router.use((req, res, next) => {
        req.kubeResourceController =
            req.kubeManager[`get${_.upperFirst(options.kubeController)}Controller`].bind(
                req.kubeManager);
        next();
    });


    router.get('/', controller.getAll.bind(controller));

    router.get('/:name', controller.describe.bind(controller));

    router.post('/', controller.create.bind(controller));

    router.delete('/:name', controller.delete.bind(controller));

    router.put('/:name', controller.update.bind(controller));

    //Non standard routes comes here
    const extraRoutes = _.get(options, 'expressController.extraRoutes', false);
    if (extraRoutes) {
        require(`./kube-native/${options.expressController.path || route}`).init(router, controller);
    }


    return router;
}


module.exports = {
    routes: _.keys(supportedRoutes).map((ep) => `kube-native/${ep}`),

    createRoute: (expressApp) => {
        _.forOwn(supportedRoutes, (options, route) => {
            options = options || {};
            expressApp.use(`/api/${route}`, initExpressController(route, options));
        });
    }
};
